@@ -1,6 +1,7 @@
 package Net::FTP::Recursive;
 
 use Net::FTP;
+use Carp;
 use Cwd 'getcwd';
 use strict;
 
@@ -8,7 +9,7 @@ use vars qw/@ISA $VERSION $file_type $dir_type $link_type/;
 use vars qw/%options %filesSeen %dirsSeen %linkMap $success/;
 
 @ISA = qw|Net::FTP|;
-$VERSION = '1.9';
+$VERSION = '2.00';
 
 ###################################################################
 # Constants for the different file types
@@ -113,11 +114,23 @@ sub _rget {
 
       $filesSeen{$filename}++ if $options{FlattenTree};
 
-      if ( $get_success and $options{RemoveRemoteFiles} ) {
-	$ftp->delete( $filename );
-	print STDERR "Deleting '$filename'.\n" if $ftp->debug;
+      if ( $options{RemoveRemoteFiles} ) {
+	if ( $options{CheckSizes} ) {
+	  if ( -e $filename and ( (-s $filename) == $file->size ) ) {
+	    $ftp->delete( $filename );
+	    print STDERR "Deleting '$filename'.\n" if $ftp->debug;
+	  } else {
+	    print STDERR "Will not delete '$filename': remote file size and local file size do not match!\n" if $ftp->debug;
+	  }
+	} else {
+	  if ( $get_success ) {
+	    $ftp->delete( $filename );
+	    print STDERR "Deleting '$filename'.\n" if $ftp->debug;
+	  } else {
+	    print STDERR "Will not delete '$filename': error retrieving file!\n" if $ftp->debug;
+	  }
+	}
       }
-
     }
 
     #if it's a directory, we have more work to do.
@@ -410,15 +423,29 @@ sub _rput {
 	$put_success = $ftp->put( $filename, "$filename.$filesSeen{$filename}");
       } else {
 	print STDERR "Sending $filename.\n" if $ftp->debug;
+
+	#I've saved $put_success here, but apparently the
+	#return val isn't very useful-can probably stop
+	#saving it
 	$put_success = $ftp->put( $filename );
       }
 
       $filesSeen{$filename}++ if $options{FlattenTree};
 
       if ( $options{RemoveLocalFiles} ) {
-	print STDERR 'Removing \'', $file->filename(),
-	  "' from the local system.\n" if $ftp->debug;
-	unlink $file->filename();
+	if ( $options{CheckSizes} ) {
+	  if ( $ftp->size($filename) == (-s $filename) ) {
+	    print STDERR 'Removing \'', $file->filename(),
+	      "' from the local system.\n" if $ftp->debug;
+	    unlink $file->filename();
+	  } else {
+	    print STDERR "Will not delete '$filename': remote file size and local file size do not match!\n" if $ftp->debug;
+	  }
+	} else { #don't care about checking the sizes
+	  print STDERR 'Removing \'', $file->filename(),
+	    "' from the local system.\n" if $ftp->debug;
+	  unlink $file->filename();
+	}
       }
 
     }
@@ -541,7 +568,9 @@ sub rdir{
 	      InitialDir => $ftp->pwd
 	     );    #setup the options
 
-  return unless $options{Filehandle};
+  unless( $options{Filehandle} ) {
+    Carp::croak("You must pass a filehandle when using rdelete/rls!");
+  }
 
   %dirsSeen = ();
   %filesSeen = ();
@@ -614,22 +643,43 @@ sub _rdir{
 	  # to this directory
 	  push @dirs, $file;
 	  $dirsSeen{$remote_abs_path}++;
-	  print STDERR qq<Mapping '$remote_abs_path' to '$dirsSeen{$remote_abs_path}'.\n>;
+	  #print STDERR qq<Mapping '$remote_abs_path' to '$dirsSeen{$remote_abs_path}'.\n>;
 
 	}
       } #end of if( $ftp->cwd( $filename ) ){
     } #end of if( $file->isSymlink() ){
     elsif ( $file->isDirectory() ) {
         push @dirs, $file;
+
+	#since we won't get to the code below, we need this
+	#code here
+	if ( $options{FilenameOnly} && $options{PrintType} ) {
+	  print $fh $remote_pwd, '/', $filename, " d\n";
+	}
+
 	next FILE if $options{FilenameOnly};
     }
 
+
     if( $options{FilenameOnly} ){
-      print $fh $remote_pwd, '/', $filename,"\n";
+      print $fh $remote_pwd, '/', $filename;
+      if ( $options{PrintType} ) {
+        my $filetype;
+        if ( $file->isSymlink() ) {
+	  $filetype = 's';
+	} elsif ( $file->isDirectory() ) {
+	  # $filetype = 'd';
+	  # we should never get here, this
+	  # is just a sanity check
+	} else {
+	  $filetype = 'f';
+        }
+        print $fh ' ', $filetype;
+      }
+      print $fh "\n";
     } else {
       print $fh $file->originalLine(), "\n";
     }
-
   }
 
   undef @files; #mark this for cleanup, it might matter
@@ -778,8 +828,8 @@ sub parse_files {
       $line =~ /^
                  (\S+)\s+ #permissions %p
                  (\d+)\s+ #link count %lc
-                 (\w+)\s+ #user owner %u
-                 (\w+)\s+ #group owner %g
+                 (\S+)\s+ #user owner %u
+                 (\S+)\s+ #group owner %g
                  (\d+)\s+ #size %s
                  (\w+\s+\w+\s+\S+)\s+ #last modification date %d
                  (.+?)\s* #filename %f
@@ -971,6 +1021,10 @@ sub filename{
   return $_[0]->{Fields}[6];
 }
 
+sub size{
+  return $_[0]->{Fields}[4];
+}
+
 sub linkName{
   return $_[0]->{Fields}[7];
 }
@@ -1111,7 +1165,16 @@ the number of times it has retrieved a file with that name.
 The optional C<RemoveRemoteFiles> argument to the function
 will allow the client to delete files from the server after
 it retrieves them.  The default behavior is to leave all
-files and directories intact.
+files and directories intact.  The default behavior for this
+is to check the return code from the FTP GET call.  If that
+is successful, it will delete the file.  C<CheckSizes> is an
+additional argument that will check the filesize of the
+local file against the file size of the remote file, and
+only if they are the same will it delete the file.  You must
+l provide the C<RemoveRemoteFiles> option in order for
+option to affect the behavior of the code.  This check will
+only be performed for regular files, not directories or
+symlinks.
 
 For the v1.6 release, I have also added some additional
 functionality that will allow the client to be more specific
@@ -1196,12 +1259,20 @@ the number of times it has retrieved a file with that name.
 The optional C<RemoveLocalFiles> argument to the function
 will allow the client to delete files from the client after
 it sends them.  The default behavior is to leave all files
-and directories intact.  This option will only attempt to
-delete files that were actually transferred, symlinks
-(unless you set SymlinkCopy and it is a plain file) will not
-be removed (and of course any non-empty directories).
+and directories intact.  This option is very unintelligent,
+it does a delete no matter what.
 
-=item rdir ( Filehandle => $fh [, FilenameOnly => 1]
+As of v1.11, there is a C<CheckSizes> option that can be
+used in conjunction with the C<RemoveLocalFiles> that will
+check the filesize of the file locally against the remote
+filesize and only delete if the two are the same.  This
+option only affects regular files, not symlinks or
+directories.  This option does not affect the normal
+behavior of C<RemoveRemoteFiles> option (ie, it will try to
+delete symlinks and directories no matter what).
+
+=item rdir ( Filehandle => $fh 
+             [, FilenameOnly => 1 [, PrintType => 1] ]
              [, ParseSub => \&yoursub ] )
 
 The recursive dir method call.  This will recursively retrieve
@@ -1215,17 +1286,29 @@ The second, optional argument, is to retrieve only the filenames
 (including path information).  The default is to display all of the
 information returned from the $ftp-dir call.
 
-This method WILL follow symlinks.  It has the same basic
+This method B<WILL> follow symlinks.  It has the same basic
 cycle-checking code that is in rget, so it should not infinitely
 loop.
 
-=item rls ( Filehandle => $fh [, ParseSub => \&yoursub] )
+The C<PrintType> argument will print either an 's', an 'f',
+or a 'd' after the filename when printed, to tell you what
+kind of file it thinks it is.  This argument must be given
+along with the FilenameOnly argument.  (Submitted by Arturas
+Slajus).
+
+=item rls ( Filehandle => $fh
+            [, PrintType => 1 ]
+            [, ParseSub => \&yoursub] )
 
 The recursive ls method call.  This will recursively
 retrieve directory contents from the server in a
 breadth-first fashion.  This is equivalent to calling
 C<$ftp->rdir( Filehandle => $fh, FilenameOnly => 1 )>.
 
+There is also a new argument to this, the C<PrintType>
+referenced in the rdir part of the documentation.  For rls,
+the FilenameOnly argument is automatically passed.
+
 =item rdelete ( [ ParseSub => \&yoursub ] )
 
 The recursive delete method call.  This will recursively
@@ -1335,17 +1418,27 @@ ftp(1), ftpd(8), RFC 959
 
 (in Chronological order, sorry if I missed anyone)
 
-Andrew Winkler - for various input into the module.
-Raj Mudaliar - documentation fix.
-Brian Reischl - for rdelete code.
-Chris Smith - for RemoveRemoteFiles code.
-Zainul Charbiwala - bug report & code to fix.
-Brian McGraw - bug report & feature request.
-Isaac Koenig - bug report
+- Andrew Winkler - for various input into the module.
+
+- Raj Mudaliar - documentation fix.
+
+- Brian Reischl - for rdelete code.
+
+- Chris Smith - for RemoveRemoteFiles code.
+
+- Zainul Charbiwala - bug report & code to fix.
+
+- Brian McGraw - bug report & feature request.
+
+- Isaac Koenig - bug report
+
+- Arturas Slajus - feature request and code for that feature request.
+
+And anyone else who gave me input on the module.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001-2003 Jeremiah Lee.
+Copyright (c) 2001-2005 Jeremiah Lee.
 
 This program is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
